# fary
> A light Nodejs middle layer framework based on Koa2, written in typescript.

## Install 
````
npm i fary
````

## Usage
````js
const createApp = require('fary')

createApp({
  middlewares: [
    async (ctx, next) => {
      ctx.body = 'Hello fary!'
    },
  ]
}, () => {
  // fary server has been started!
})
````
Put a `fary.config.js` in your root of project, the content like:
````js
module.exports = {
  port: 9008,
  proxy: {
    '/api': 'https://github.com',
  },
}
````
*Note: with this simple example, no request will be proxied, because all requests will be responded `Hello fary!`*

## Features
1. request/response log with [koa-logger](https://github.com/koajs/logger)
2. compress with [koa-compress](https://github.com/koajs/compress)
3. pass your middlewares to do whatever you want.
4. http proxy with [koa-better-http-proxy](https://github.com/nsimmons/koa-better-http-proxy)

## Proxy
Support multi proxy targets.
Assume the `fary.config.js` is:
```js
module.exports = {
  port: 9008,
  proxy: {
    '/api/github': 'https://github.com',
    '/api/node': 'https://nodejs.org',
  },
}
```
The entry:
```js
require('fary')({
  middlewares: [
    // some middlewares(for example: routes), but do not match `/api/github` or `/api/node` requests!
  ]
})
```
Then `/api/github/explore` will be proxied to `https://github.com/explore`, `/api/node/en/about` will be proxied to `https://nodejs.org/en/about`.
That is: remove the first matched proxy key(here is `/api/github` or `/api/node`) with which the current request url starts, then append the remaining to the target.

*Make sure the request can reach downstreamly the proxy middleware(the last one).*
