"use strict";
const log = require("fary-log");
const Koa = require("koa");
const koaCompress = require("koa-compress");
const koaLogger = require("koa-logger");
const errorMiddleware = require("./middlewares/error");
const proxyMiddleware = require("./middlewares/proxy");
const checkProxy = require("./utils/check-proxy");
const ENV = require("./utils/env");
const serverConfig = require("./utils/server-config");
const { IS_DEV } = ENV;
const logger = log('fary:app');
const serverConfigValue = serverConfig.value;
const DEFAULT_OPTIONS = {
    middlewares: [],
};
/**
 * createApp
 * @param {Object} options - options
 * @param {Function} onStart - callback after server start
 * @return {Koa} app - serverApp
 */
function createApp(options, onStart = () => void 0) {
    options = Object.assign({}, DEFAULT_OPTIONS, options);
    if (IS_DEV) {
        checkProxy(serverConfigValue.proxy);
    }
    logger.log(`Debug mode:`, IS_DEV);
    logger.log(`Proxy mapping:`, serverConfigValue.proxy);
    const app = new Koa();
    // trust proxy
    app.proxy = true;
    const middlewares = [
        // request/response log
        koaLogger(),
        // compress the response
        koaCompress(),
        // error handler
        errorMiddleware(),
        ...options.middlewares,
        // proxy handler
        proxyMiddleware(),
    ];
    middlewares.forEach((middleware) => {
        if (!middleware) {
            return;
        }
        app.use(middleware);
    });
    app.on('error', (err) => {
        logger.error(`Server error: \n ${err.stack}`);
    });
    app.listen(serverConfigValue.port, () => {
        logger.info(`Koa server is listening at ${serverConfigValue.port}`);
        onStart();
    });
    process.on('SIGINT', () => {
        process.exit();
    });
    return app;
}
module.exports = createApp;
//# sourceMappingURL=index.js.map